
// Platform-dependent constants

// Largest and smallest values of arithmetic types

#include <values.h>

inline void mostneg(int* res) {
  *res = (- MAXINT) - 1 ;
}

inline void mostneg(float* res) {
  *res = - MAXFLOAT ;
}

inline void mostneg(double* res) {
  *res = - MAXDOUBLE ;
}

inline void mostpos(int* res) {
  *res = MAXINT ;
}

inline void mostpos(float* res) {
  *res = MAXFLOAT ;
}

inline void mostpos(double* res) {
  *res = MAXDOUBLE ;
}


// Complex numbers

#include <math.h>

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
//                                                                     //
// class complex.                                                      //
//                                                                     //
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

class complex {
public:
  complex(double _rpart, double _ipart) : rpart(_rpart), ipart(_ipart) {}
  complex(double _rpart) : rpart(_rpart), ipart(0) {}
  complex(int _rpart) : rpart(_rpart), ipart(0) {}
  complex() : rpart(0), ipart(0) {}

  complex(const complex& c) : rpart(c.rpart), ipart(c.ipart) {}

  complex& operator=(const complex& c) {
    rpart = c.rpart ; ipart = c.ipart ; return *this ;
  }

  complex& operator+=(const complex& c) {
    rpart += c.rpart ; ipart += c.ipart ; return *this ;
  }

  complex& operator-=(const complex& c) {
    rpart -= c.rpart ; ipart -= c.ipart ; return *this ;
  }

  complex& operator*=(const complex& c) {
    const float rnew = rpart * c.rpart - ipart * c.ipart ;
    const float inew = ipart * c.rpart + rpart * c.ipart ;
    rpart = rnew ;
    ipart = inew ;
    return *this ;
  }

  complex& operator/=(const complex& c) {
    const float div = c.rpart * c.rpart + c.ipart * c.ipart ;
    const float rnew = (rpart * c.rpart + ipart * c.ipart) / div ;
    const float inew = (ipart * c.rpart - rpart * c.ipart) / div ;
    rpart = rnew ;
    ipart = inew ;
    return *this ;
  }

  complex operator+(const complex& c) const {
    return complex(rpart + c.rpart, ipart + c.ipart) ;
  }

  complex operator-(const complex& c) const {
    return complex(rpart - c.rpart, ipart - c.ipart) ;
  }

  complex operator*(const complex& c) const {
    return complex(rpart * c.rpart - ipart * c.ipart,
                   ipart * c.rpart + rpart * c.ipart) ;
  }

  complex operator/(const complex& c) const {
    const float div = c.rpart * c.rpart + c.ipart * c.ipart ;
    return complex((rpart * c.rpart + ipart * c.ipart) / div,
                   (ipart * c.rpart - rpart * c.ipart) / div) ;
  }

  float re() const {return rpart ; }
  float im() const {return ipart ; }

  float abs() const {return sqrt(rpart * rpart + ipart * ipart) ; }

private:
  float rpart, ipart ;
} ;

inline complex operator+(const double d, const complex& c) {
  return complex(d) + c ;
}

inline complex operator+(const int i, const complex& c) {
  return complex(i) + c ;
}

inline complex operator-(const double d, const complex& c) {
  return complex(d) - c ;
}

inline complex operator-(const int i, const complex& c) {
  return complex(i) - c ;
}

inline complex operator*(const double d, const complex& c) {
  return complex(d) * c ;
}

inline complex operator*(const int i, const complex& c) {
  return complex(i) * c ;
}

inline complex operator/(const double d, const complex& c) {
  return complex(d) / c ;
}

inline complex operator/(const int i, const complex& c) {
  return complex(i) / c ;
}


#define COMPLEX complex  // Allow for other names, eg ANSI complex template.


// Size of Fortran data types.

#define SIZE_INTEGER sizeof(int)
#define SIZE_REAL    sizeof(float)
#define SIZE_DP      sizeof(double)
#define SIZE_COMPLEX (2 * sizeof(float))
#define SIZE_LOGICAL sizeof(int)


